@@ -32,7 +32,7 @@ use ockam_node::Context;
 /// The same `TcpTransport` can also bind to multiple ports.
 ///
 /// ```rust
-/// # use ockam_transport_tcp::TcpTransport;
+/// use ockam_transport_tcp::TcpTransport;
 /// # use ockam_node::Context;
 /// # use ockam_core::Result;
 /// # async fn test(ctx: Context) -> Result<()> {
@@ -79,10 +79,15 @@ impl TcpTransport {
     /// tcp.connect("127.0.0.1:5000").await?; // and connect to port 5000
     /// # Ok(()) }
     /// ```
-    pub async fn connect<S: AsRef<str>>(&self, peer: S) -> Result<()> {
+    pub async fn connect<S: AsRef<str>>(&self, peer: S) -> Result<Address> {
         self.router_handle.connect(peer.as_ref()).await
     }
 
+    /// Disconnect from peer
+    pub async fn disconnect<S: AsRef<str>>(&self, peer: S) -> Result<()> {
+        self.router_handle.disconnect(peer.as_ref()).await
+    }
+
     /// Start listening to incoming connections on an existing transport
     /// ```rust
     /// use ockam_transport_tcp::TcpTransport;
