@@ -1,5 +1,5 @@
 use crate::{TcpRouterHandle, TcpSendWorker};
-use ockam_core::async_trait;
+use ockam_core::{async_trait, AsyncTryClone};
 use ockam_core::{Address, Processor, Result};
 use ockam_node::Context;
 use ockam_transport_core::TransportError;
@@ -18,7 +18,7 @@ impl TcpListenProcessor {
         router_handle: TcpRouterHandle,
         addr: SocketAddr,
     ) -> Result<()> {
-        let waddr = Address::random(0);
+        let waddr = Address::random_local();
 
         debug!("Binding TcpListener to {}", addr);
         let inner = TcpListener::bind(addr)
@@ -48,8 +48,9 @@ impl Processor for TcpListenProcessor {
         // Wait for an incoming connection
         let (stream, peer) = self.inner.accept().await.map_err(TransportError::from)?;
 
+        let handle_clone = self.router_handle.async_try_clone().await?;
         // And spawn a connection worker for it
-        let pair = TcpSendWorker::start_pair(ctx, Some(stream), peer, vec![]).await?;
+        let pair = TcpSendWorker::start_pair(ctx, handle_clone, Some(stream), peer, vec![]).await?;
 
         // Register the connection with the local TcpRouter
         self.router_handle.register(&pair).await?;
